@@ -6,7 +6,8 @@ from contextlib import contextmanager
 from itertools import chain
 from pathlib import Path
 from tempfile import NamedTemporaryFile
-from textwrap import dedent
+from textwrap import dedent, indent
+from typing import Any
 
 try:
     from unittest import mock
@@ -94,6 +95,43 @@ def test_package_dag_filter():
         dag_to_dict(t.filter({"d"}, {"D", "e"}))
 
 
+@pytest.fixture(scope="session")
+def t_fnmatch() -> Any:
+    return mock_package_dag(
+        {
+            ("a.a", "1"): [("a.b", []), ("a.c", [])],
+            ("a.b", "1"): [("a.c", [])],
+            ("b.a", "1"): [("b.b", [])],
+            ("b.b", "1"): [("a.b", [])],
+        }
+    )
+
+
+def test_package_dag_filter_fnmatch_include_a(t_fnmatch: Any) -> None:
+    # test include for a.*in the result we got only a.* nodes
+    graph = dag_to_dict(t_fnmatch.filter({"a.*"}, None))
+    assert graph == {"a.a": ["a.b", "a.c"], "a.b": ["a.c"]}
+
+
+def test_package_dag_filter_fnmatch_include_b(t_fnmatch: Any) -> None:
+    # test include for b.*, which has a.b and a.c in tree, but not a.a
+    # in the result we got the b.* nodes plus the a.b node as child in the tree
+    graph = dag_to_dict(t_fnmatch.filter({"b.*"}, None))
+    assert graph == {"b.a": ["b.b"], "b.b": ["a.b"], "a.b": ["a.c"]}
+
+
+def test_package_dag_filter_fnmatch_exclude_c(t_fnmatch: Any) -> None:
+    # test exclude for b.* in the result we got only a.* nodes
+    graph = dag_to_dict(t_fnmatch.filter(None, {"b.*"}))
+    assert graph == {"a.a": ["a.b", "a.c"], "a.b": ["a.c"]}
+
+
+def test_package_dag_filter_fnmatch_exclude_a(t_fnmatch: Any) -> None:
+    # test exclude for a.* in the result we got only b.* nodes
+    graph = dag_to_dict(t_fnmatch.filter(None, {"a.*"}))
+    assert graph == {"b.a": ["b.b"], "b.b": []}
+
+
 def test_package_dag_reverse():
     t1 = t.reverse()
     expected = {"a": [], "b": ["a", "f"], "c": ["a"], "d": ["b", "c"], "e": ["c", "d", "g"], "f": ["g"], "g": []}
@@ -327,33 +365,64 @@ def randomized_dag_copy(t):
 def test_render_mermaid():
     """Check both the sorted and randomized package tree produces the same sorted Mermaid output.
 
+    Rendering a reverse dependency tree should produce the same set of nodes. Edges should have
+    the same version spec label, but be resorted after swapping node positions.
+
     `See how this renders
     <https://mermaid.ink/img/pako:eNp9kcluwjAURX_FeutgeUhCErWs-IN21boL4yFEzYAyqKWIf-9LCISyqFf2ufe-QT6BaayDDHzZfJm9bnvyulU1wWNK3XVb50lVdF1R56Tr2-bTrazu0NfqY0aii1O_K9BK1ZKGlCn4uNAd0h1SQSXlN2qQGqQR5ezObBHbizm6QYfQIWSUi7sSHrGf2i0sR5Yji2lCZWsWQZPViijYPAs69cPnhuwetIiux1qTZubpl5xkwZOgoZgNdl7karhON4nuQRzTf3N2yaW3geaYX2L8cdj8n9xNk3dLegigcm2lC4v_exqdCvq9q5yCDK_WeT2UvQJVn9Gqh755OdYGsr4dXADDwerebQudt7qCzOuyQ3rQ9VvTVFcTPiE7wTdkgkVUSiHjlLOERUkYB3AcMeXrhKUp53GYcJ6KcwA_UwVGo_MvVqym_A?type=png)](https://mermaid.live/edit#pako:eNp9kcluwjAURX_FeutgeUhCErWs-IN21boL4yFEzYAyqKWIf-9LCISyqFf2ufe-QT6BaayDDHzZfJm9bnvyulU1wWNK3XVb50lVdF1R56Tr2-bTrazu0NfqY0aii1O_K9BK1ZKGlCn4uNAd0h1SQSXlN2qQGqQR5ezObBHbizm6QYfQIWSUi7sSHrGf2i0sR5Yji2lCZWsWQZPViijYPAs69cPnhuwetIiux1qTZubpl5xkwZOgoZgNdl7karhON4nuQRzTf3N2yaW3geaYX2L8cdj8n9xNk3dLegigcm2lC4v_exqdCvq9q5yCDK_WeT2UvQJVn9Gqh755OdYGsr4dXADDwerebQudt7qCzOuyQ3rQ9VvTVFcTPiE7wTdkgkVUSiHjlLOERUkYB3AcMeXrhKUp53GYcJ6KcwA_UwVGo_MvVqym_A>`_.
     """
+
+    nodes = dedent(
+        """\
+        flowchart TD
+            classDef missing stroke-dasharray: 5
+            a["a\\n3.4.0"]
+            b["b\\n2.3.1"]
+            c["c\\n5.10.0"]
+            d["d\\n2.35"]
+            e["e\\n0.12.1"]
+            f["f\\n3.1"]
+            g["g\\n6.8.3rc1"]
+        """
+    )
+    dependency_edges = indent(
+        dedent(
+            """\
+            a -- ">=2.0.0" --> b
+            a -- ">=5.7.1" --> c
+            b -- ">=2.30,<2.42" --> d
+            c -- ">=0.12.1" --> e
+            c -- ">=2.30" --> d
+            d -- ">=0.9.0" --> e
+            f -- ">=2.1.0" --> b
+            g -- ">=0.9.0" --> e
+            g -- ">=3.0.0" --> f
+        """
+        ),
+        " " * 4,
+    ).rstrip()
+    reverse_dependency_edges = indent(
+        dedent(
+            """\
+            b -- ">=2.0.0" --> a
+            b -- ">=2.1.0" --> f
+            c -- ">=5.7.1" --> a
+            d -- ">=2.30" --> c
+            d -- ">=2.30,<2.42" --> b
+            e -- ">=0.12.1" --> c
+            e -- ">=0.9.0" --> d
+            e -- ">=0.9.0" --> g
+            f -- ">=3.0.0" --> g
+        """
+        ),
+        " " * 4,
+    ).rstrip()
+
     for package_tree in (t, randomized_dag_copy(t)):
         output = p.render_mermaid(package_tree)
-        assert output == dedent(
-            """\
-            flowchart TD
-                classDef missing stroke-dasharray: 5
-                a["a\\n3.4.0"]
-                b["b\\n2.3.1"]
-                c["c\\n5.10.0"]
-                d["d\\n2.35"]
-                e["e\\n0.12.1"]
-                f["f\\n3.1"]
-                g["g\\n6.8.3rc1"]
-                a -- ">=2.0.0" --> b
-                a -- ">=5.7.1" --> c
-                b -- ">=2.30,<2.42" --> d
-                c -- ">=0.12.1" --> e
-                c -- ">=2.30" --> d
-                d -- ">=0.9.0" --> e
-                f -- ">=2.1.0" --> b
-                g -- ">=0.9.0" --> e
-                g -- ">=3.0.0" --> f
-            """
-        )
+        assert output.rstrip() == nodes + dependency_edges
+        reversed_output = p.render_mermaid(package_tree.reverse())
+        assert reversed_output.rstrip() == nodes + reverse_dependency_edges
 
 
 def test_mermaid_reserved_ids():
